@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: player.h 1.1 2008/01/12 16:21:57 kls Exp $
+ * $Id: player.h 5.1 2025/03/02 11:03:35 kls Exp $
  */
 
 #ifndef _PLAYER_H
@@ -35,11 +35,12 @@ private:
   void NextPicture(int Direction);
   void NextDirectory(int Direction);
   void DisplayCaption(void);
-  virtual void Hide(void) {}
+  virtual void Hide(void) override {}
 public:
   cPictureControl(cPictureEntry *Pictures, const cPictureEntry *PictureEntry, bool SlideShow = false);
-  virtual ~cPictureControl();
-  virtual eOSState ProcessKey(eKeys Key);
+  virtual ~cPictureControl() override;
+  virtual cString GetHeader(void) override;
+  virtual eOSState ProcessKey(eKeys Key) override;
   static bool Active(void) { return active > 0; }
   static const char *LastDisplayed(void);
   };
