@@ -3,13 +3,13 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: status.c 1.8 2005/12/31 15:19:45 kls Exp $
+ * $Id: status.c 5.6 2025/03/02 21:02:12 kls Exp $
  */
 
 #include <vdr/plugin.h>
 #include <vdr/status.h>
 
-static const char *VERSION        = "0.2.0";
+static const char *VERSION        = "2.6.2";
 static const char *DESCRIPTION    = "Status monitor test";
 static const char *MAINMENUENTRY  = NULL;
 
@@ -17,23 +17,33 @@ static const char *MAINMENUENTRY  = NULL;
 
 class cStatusTest : public cStatus {
 protected:
-  virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
-  virtual void Recording(const cDevice *Device, const char *Name, const char *FileName, bool On);
-  virtual void Replaying(const cControl *Control, const char *Name, const char *FileName, bool On);
-  virtual void SetVolume(int Volume, bool Absolute);
-  virtual void OsdClear(void);
-  virtual void OsdTitle(const char *Title);
-  virtual void OsdStatusMessage(const char *Message);
-  virtual void OsdHelpKeys(const char *Red, const char *Green, const char *Yellow, const char *Blue);
-  virtual void OsdCurrentItem(const char *Text);
-  virtual void OsdTextItem(const char *Text, bool Scroll);
-  virtual void OsdChannel(const char *Text);
-  virtual void OsdProgramme(time_t PresentTime, const char *PresentTitle, const char *PresentSubtitle, time_t FollowingTime, const char *FollowingTitle, const char *FollowingSubtitle);
+  virtual void TimerChange(const cTimer *Timer, eTimerChange Change) override;
+  virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber, bool LiveView) override;
+  virtual void Recording(const cDevice *Device, const char *Name, const char *FileName, bool On) override;
+  virtual void Replaying(const cControl *Control, const char *Name, const char *FileName, bool On) override;
+  virtual void SetVolume(int Volume, bool Absolute) override;
+  virtual void SetAudioTrack(int Index, const char * const *Tracks) override;
+  virtual void SetAudioChannel(int AudioChannel) override;
+  virtual void SetSubtitleTrack(int Index, const char * const *Tracks) override;
+  virtual void OsdClear(void) override;
+  virtual void OsdTitle(const char *Title) override;
+  virtual void OsdStatusMessage(eMessageType Type, const char *Message) override;
+  virtual void OsdHelpKeys(const char *Red, const char *Green, const char *Yellow, const char *Blue) override;
+  virtual void OsdItem(const char *Text, int Index, bool Selectable) override;
+  virtual void OsdCurrentItem(const char *Text, int Index) override;
+  virtual void OsdTextItem(const char *Text, bool Scroll) override;
+  virtual void OsdChannel(const char *Text) override;
+  virtual void OsdProgramme(time_t PresentTime, const char *PresentTitle, const char *PresentSubtitle, time_t FollowingTime, const char *FollowingTitle, const char *FollowingSubtitle) override;
   };
 
-void cStatusTest::ChannelSwitch(const cDevice *Device, int ChannelNumber)
+void cStatusTest::TimerChange(const cTimer *Timer, eTimerChange Change)
 {
-  dsyslog("status: cStatusTest::ChannelSwitch  %d %d", Device->CardIndex(), ChannelNumber);
+  dsyslog("status: cStatusTest::TimerChange  %s %d", Timer ? *Timer->ToText(true) : "-", Change);
+}
+
+void cStatusTest::ChannelSwitch(const cDevice *Device, int ChannelNumber, bool LiveView)
+{
+  dsyslog("status: cStatusTest::ChannelSwitch  %d %d %d", Device->CardIndex(), ChannelNumber, LiveView);
 }
 
 void cStatusTest::Recording(const cDevice *Device, const char *Name, const char *FileName, bool On)
@@ -51,6 +61,21 @@ void cStatusTest::SetVolume(int Volume, bool Absolute)
   dsyslog("status: cStatusTest::SetVolume  %d %d", Volume, Absolute);
 }
 
+void cStatusTest::SetAudioTrack(int Index, const char * const *Tracks)
+{
+  dsyslog("status: cStatusTest::SetAudioTrack  %d %s", Index, Tracks[Index]);
+}
+
+void cStatusTest::SetAudioChannel(int AudioChannel)
+{
+  dsyslog("status: cStatusTest::SetAudioChannel  %d", AudioChannel);
+}
+
+void cStatusTest::SetSubtitleTrack(int Index, const char * const *Tracks)
+{
+  dsyslog("status: cStatusTest::SetSubtitleTrack  %d %s", Index, Tracks[Index]);
+}
+
 void cStatusTest::OsdClear(void)
 {
   dsyslog("status: cStatusTest::OsdClear");
@@ -61,9 +86,9 @@ void cStatusTest::OsdTitle(const char *Title)
   dsyslog("status: cStatusTest::OsdTitle '%s'", Title);
 }
 
-void cStatusTest::OsdStatusMessage(const char *Message)
+void cStatusTest::OsdStatusMessage(eMessageType Type, const char *Message)
 {
-  dsyslog("status: cStatusTest::OsdStatusMessage '%s'", Message);
+  dsyslog("status: cStatusTest::OsdStatusMessage %d '%s'", Type, Message);
 }
 
 void cStatusTest::OsdHelpKeys(const char *Red, const char *Green, const char *Yellow, const char *Blue)
@@ -71,9 +96,14 @@ void cStatusTest::OsdHelpKeys(const char *Red, const char *Green, const char *Ye
   dsyslog("status: cStatusTest::OsdHelpKeys %s - %s - %s - %s", Red, Green, Yellow, Blue);
 }
 
-void cStatusTest::OsdCurrentItem(const char *Text)
+void cStatusTest::OsdItem(const char *Text, int Index, bool Selected)
 {
-  dsyslog("status: cStatusTest::OsdCurrentItem %s", Text);
+  dsyslog("status: cStatusTest::OsdItem  %s %d %d", Text, Index, Selected);
+}
+
+void cStatusTest::OsdCurrentItem(const char *Text, int Index)
+{
+  dsyslog("status: cStatusTest::OsdCurrentItem %s %d", Text, Index);
 }
 
 void cStatusTest::OsdTextItem(const char *Text, bool Scroll)
@@ -107,17 +137,17 @@ private:
   cStatusTest *statusTest;
 public:
   cPluginStatus(void);
-  virtual ~cPluginStatus();
-  virtual const char *Version(void) { return VERSION; }
-  virtual const char *Description(void) { return DESCRIPTION; }
-  virtual const char *CommandLineHelp(void);
-  virtual bool ProcessArgs(int argc, char *argv[]);
-  virtual bool Start(void);
-  virtual void Housekeeping(void);
-  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
-  virtual cOsdObject *MainMenuAction(void);
-  virtual cMenuSetupPage *SetupMenu(void);
-  virtual bool SetupParse(const char *Name, const char *Value);
+  virtual ~cPluginStatus() override;
+  virtual const char *Version(void) override { return VERSION; }
+  virtual const char *Description(void) override { return DESCRIPTION; }
+  virtual const char *CommandLineHelp(void) override;
+  virtual bool ProcessArgs(int argc, char *argv[]) override;
+  virtual bool Start(void) override;
+  virtual void Housekeeping(void) override;
+  virtual const char *MainMenuEntry(void) override { return MAINMENUENTRY; }
+  virtual cOsdObject *MainMenuAction(void) override;
+  virtual cMenuSetupPage *SetupMenu(void) override;
+  virtual bool SetupParse(const char *Name, const char *Value) override;
   };
 
 cPluginStatus::cPluginStatus(void)
