@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 5.34 2025/07/24 13:43:47 kls Exp $
+ * $Id: config.h 5.37 2026/01/21 21:19:59 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "2.7.7"
-#define VDRVERSNUM   20707  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "2.7.8"
+#define VDRVERSNUM   20708  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION      "9"
-#define APIVERSNUM   30009
+#define APIVERSION     "10"
+#define APIVERSNUM   30010
 
 // When loading plugins, VDR searches files by their APIVERSION, which
 // is different from VDRVERSION. APIVERSION is a plain number, incremented
@@ -45,6 +45,7 @@
 #define IDLEPRIORITY      (MINPRIORITY - 1)  // priority of an idle device
 #define MAXLIFETIME       99
 #define DEFINSTRECTIME    180 // default instant recording time (minutes)
+#define DEFRETENTIONTIME  0   // default deleted recording retention time (days)
 
 #define TIMERMACRO_TITLE    "TITLE"
 #define TIMERMACRO_EPISODE  "EPISODE"
@@ -312,6 +313,7 @@ public:
   int ChannelEntryTimeout;
   int RcRepeatDelay;
   int RcRepeatDelta;
+  int DeleteRetention;
   int DefaultPriority, DefaultLifetime;
   int RecordKeyHandling;
   int PauseKeyHandling;
