@@ -8,13 +8,14 @@
  *
  * parts of this file are derived from the OMS program.
  *
- * $Id: dvbspu.h 2.4 2009/12/13 12:07:00 kls Exp $
+ * $Id: dvbspu.h 5.1 2025/03/03 13:12:16 kls Exp $
  */
 
 #ifndef __DVBSPU_H
 #define __DVBSPU_H
 
 #include <inttypes.h>
+#include "device.h"
 #include "osd.h"
 #include "spu.h"
 #include "thread.h"
@@ -32,6 +33,9 @@ typedef struct sDvbSpuRect {
     int x1, y1;
     int x2, y2;
 
+    sDvbSpuRect(void) {
+        x1 = y1 = x2 = y2 = 0;
+    };
     int width() const {
         return x2 - x1 + 1;
     };
@@ -97,13 +101,13 @@ class cDvbSpuDecoder:public cSpuDecoder {
     uint8_t *spu;
     uint32_t spupts;
     bool clean;
-    bool ready;
     bool restricted_osd;
 
     enum spFlag { spNONE, spHIDE, spSHOW, spMENU };
     spFlag state;
 
-     cSpuDecoder::eScaleMode scaleMode;
+    cSpuDecoder::eScaleMode scaleMode;
+    double xscaling, yscaling;
 
     //highligh area
     bool highlight;
@@ -130,6 +134,7 @@ class cDvbSpuDecoder:public cSpuDecoder {
         return ((spu[0] << 8) | spu[1]);
     };
 
+    void SetSpuScaling(void);
     sDvbSpuRect CalcAreaSize(sDvbSpuRect fgsize, cBitmap *fgbmp, sDvbSpuRect bgsize, cBitmap *bgbmp);
     int CalcAreaBpp(cBitmap *fgbmp, cBitmap *bgbmp);
 
@@ -200,4 +205,4 @@ inline uint32_t cDvbSpuPalette::getColor(uint8_t idx, uint8_t trans) const
     return palette[idx] | ((trans == 0x0f) ? 0xff000000 : (trans << 28));
 }
 
-#endif                          // __DVBSPU_H
+#endif //__DVBSPU_H
