@@ -4,9 +4,9 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * Original author: Marco Schller <marco@lordzodiac.de>
+ * Original author: Marco Schluessler <marco@lordzodiac.de>
  *
- * $Id: dvbsubtitle.h 2.5 2011/03/12 14:03:42 kls Exp $
+ * $Id: dvbsubtitle.h 5.3 2025/03/28 22:49:17 kls Exp $
  */
 
 #ifndef __DVBSUBTITLE_H
@@ -35,18 +35,26 @@ private:
   int windowHeight;
   int osdDeltaX;
   int osdDeltaY;
-  double osdFactor;
+  double osdFactorX;
+  double osdFactorY;
+  int retention;
+  bool visible;
+  uint64_t endVisible;
   cList<cDvbSubtitlePage> *pages;
   cList<cDvbSubtitleBitmaps> *bitmaps;
-  tColor yuv2rgb(int Y, int Cb, int Cr);
+  cDvbSubtitleBitmaps *current;
+  cDvbSubtitlePage *GetPageById(int PageId, bool New = false);
   void SetOsdData(void);
   bool AssertOsd(void);
   int ExtractSegment(const uchar *Data, int Length, int64_t Pts);
+  int ExtractPgsSegment(const uchar *Data, int Length, int64_t Pts);
   void FinishPage(cDvbSubtitlePage *Page);
 public:
   cDvbSubtitleConverter(void);
-  virtual ~cDvbSubtitleConverter();
-  void Action(void);
+  virtual ~cDvbSubtitleConverter() override;
+  virtual void Action(void) override;
+  void SetVisible(bool Visible);
+  void SetTempVisible(void);
   void Reset(void);
   void Freeze(bool Status) { frozen = Status; }
   int ConvertFragments(const uchar *Data, int Length); // for legacy PES recordings
