@@ -6,7 +6,7 @@
  *
  * BiDi support by Osama Alrawab <alrawab@hotmail.com> @2008 Tripoli-Libya.
  *
- * $Id: font.c 2.7 2011/02/26 12:09:18 kls Exp $
+ * $Id: font.c 5.4 2025/03/02 11:03:35 kls Exp $
  */
 
 #include "font.h"
@@ -49,7 +49,7 @@ private:
   cVector<tKerning> kerningCache;
 public:
   cGlyph(uint CharCode, FT_GlyphSlotRec_ *GlyphData);
-  virtual ~cGlyph();
+  virtual ~cGlyph() override;
   uint CharCode(void) const { return charCode; }
   uchar *Bitmap(void) const { return bitmap; }
   int AdvanceX(void) const { return advanceX; }
@@ -74,7 +74,8 @@ cGlyph::cGlyph(uint CharCode, FT_GlyphSlotRec_ *GlyphData)
   rows = GlyphData->bitmap.rows;
   pitch = GlyphData->bitmap.pitch;
   bitmap = MALLOC(uchar, rows * pitch);
-  memcpy(bitmap, GlyphData->bitmap.buffer, rows * pitch);
+  if (int bytes = rows * pitch)
+     memcpy(bitmap, GlyphData->bitmap.buffer, bytes);
 }
 
 cGlyph::~cGlyph()
@@ -100,6 +101,7 @@ class cFreetypeFont : public cFont {
 private:
   cString fontName;
   int size;
+  int width;
   int height;
   int bottom;
   FT_Library library; ///< Handle to library
@@ -111,20 +113,22 @@ private:
   cGlyph* Glyph(uint CharCode, bool AntiAliased = false) const;
 public:
   cFreetypeFont(const char *Name, int CharHeight, int CharWidth = 0);
-  virtual ~cFreetypeFont();
-  virtual const char *FontName(void) const { return fontName; }
-  virtual int Size(void) const { return size; }
-  virtual int Width(uint c) const;
-  virtual int Width(const char *s) const;
-  virtual int Height(void) const { return height; }
-  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const;
-  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const;
+  virtual ~cFreetypeFont() override;
+  virtual const char *FontName(void) const override { return fontName; }
+  virtual int Size(void) const override { return size; }
+  virtual int Width(void) const override { return width; }
+  virtual int Width(uint c) const override;
+  virtual int Width(const char *s) const override;
+  virtual int Height(void) const override { return height; }
+  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override;
+  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override;
   };
 
 cFreetypeFont::cFreetypeFont(const char *Name, int CharHeight, int CharWidth)
 {
   fontName = Name;
   size = CharHeight;
+  width = CharWidth;
   height = 0;
   bottom = 0;
   int error = FT_Init_FreeType(&library);
@@ -140,7 +144,7 @@ cFreetypeFont::cFreetypeFont(const char *Name, int CharHeight, int CharWidth)
                if (!error) {
                   error = FT_Render_Glyph(face->glyph, FT_RENDER_MODE_NORMAL);
                   if (!error) {
-                     if (face->glyph->bitmap.rows-face->glyph->bitmap_top > bottom)
+                     if (int(face->glyph->bitmap.rows-face->glyph->bitmap_top) > bottom)
                         bottom = face->glyph->bitmap.rows-face->glyph->bitmap_top;
                      }
                   else
@@ -382,12 +386,17 @@ void cFreetypeFont::DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColo
 // A dummy font, in case there are no fonts installed:
 
 class cDummyFont : public cFont {
+private:
+  int height;
+  int width;
 public:
-  virtual int Width(uint c) const { return 10; }
-  virtual int Width(const char *s) const { return 50; }
-  virtual int Height(void) const { return 20; }
-  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const {}
-  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const {};
+  cDummyFont(int CharHeight, int CharWidth) { height = CharHeight; width = CharWidth; }
+  virtual int Width(void) const override { return width ? width : height; }
+  virtual int Width(uint c) const override { return width ? width : height; }
+  virtual int Width(const char *s) const override { return width ? width : height; }
+  virtual int Height(void) const override { return height; }
+  virtual void DrawText(cBitmap *Bitmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override {}
+  virtual void DrawText(cPixmap *Pixmap, int x, int y, const char *s, tColor ColorFg, tColor ColorBg, int Width) const override {}
   };
 
 // --- cFont -----------------------------------------------------------------
@@ -396,11 +405,8 @@ cFont *cFont::fonts[eDvbFontSize] = { NULL };
 
 void cFont::SetFont(eDvbFont Font, const char *Name, int CharHeight)
 {
-  cFont *f = CreateFont(Name, min(max(CharHeight, MINFONTSIZE), MAXFONTSIZE));
-  if (!f || !f->Height())
-     f = new cDummyFont;
   delete fonts[Font];
-  fonts[Font] = f;
+  fonts[Font] = CreateFont(Name, constrain(CharHeight, MINFONTSIZE, MAXFONTSIZE));
 }
 
 const cFont *cFont::GetFont(eDvbFont Font)
@@ -412,7 +418,7 @@ const cFont *cFont::GetFont(eDvbFont Font)
   if (!fonts[Font]) {
      switch (Font) {
        case fontOsd: SetFont(Font, Setup.FontOsd, Setup.FontOsdSize); break;
-       case fontSml: SetFont(Font, Setup.FontSml, Setup.FontSmlSize); break;
+       case fontSml: SetFont(Font, Setup.FontSml, min(Setup.FontSmlSize, Setup.FontOsdSize)); break;
        case fontFix: SetFont(Font, Setup.FontFix, Setup.FontFixSize); break;
        default: esyslog("ERROR: unknown Font %d (%s %d)", Font, __FUNCTION__, __LINE__);
        }
@@ -423,9 +429,12 @@ const cFont *cFont::GetFont(eDvbFont Font)
 cFont *cFont::CreateFont(const char *Name, int CharHeight, int CharWidth)
 {
   cString fn = GetFontFileName(Name);
-  if (*fn)
-     return new cFreetypeFont(fn, CharHeight, CharWidth);
-  return NULL;
+  cFont *f = *fn ? new cFreetypeFont(fn, CharHeight, CharWidth) : NULL;
+  if (!f || !f->Height()) {
+     delete f;
+     f = new cDummyFont(CharHeight, CharWidth);
+     }
+  return f;
 }
 
 bool cFont::GetAvailableFontNames(cStringList *FontNames, bool Monospaced)
@@ -482,7 +491,8 @@ cString cFont::GetFontFileName(const char *FontName)
      FcPatternAddBool(pat, FC_SCALABLE, FcTrue);
      FcConfigSubstitute(NULL, pat, FcMatchPattern);
      FcDefaultSubstitute(pat);
-     FcFontSet *fontset = FcFontSort(NULL, pat, FcFalse, NULL, NULL);
+     FcResult fresult;
+     FcFontSet *fontset = FcFontSort(NULL, pat, FcFalse, NULL, &fresult);
      if (fontset) {
         for (int i = 0; i < fontset->nfont; i++) {
             FcBool scalable;
@@ -508,25 +518,27 @@ cString cFont::GetFontFileName(const char *FontName)
 #ifdef BIDI
 cString cFont::Bidi(const char *Ltr)
 {
-  fribidi_set_mirroring(true);
-  fribidi_set_reorder_nsm(false);
-  FriBidiCharSet fribidiCharset = FRIBIDI_CHAR_SET_UTF8;
-  int LtrLen = strlen(Ltr);
-  FriBidiCharType Base = FRIBIDI_TYPE_L;
-  FriBidiChar *Logical = MALLOC(FriBidiChar, LtrLen + 1) ;
-  int RtlLen = fribidi_charset_to_unicode(fribidiCharset, const_cast<char *>(Ltr), LtrLen, Logical);
-  FriBidiChar *Visual = MALLOC(FriBidiChar, LtrLen + 1) ;
-  char *Rtl = NULL;
-  bool ok = fribidi_log2vis(Logical, RtlLen, &Base, Visual, NULL, NULL, NULL);
-  if (ok) {
-     fribidi_remove_bidi_marks(Visual, RtlLen, NULL, NULL, NULL);
-     Rtl = MALLOC(char, RtlLen * 4);
-     fribidi_unicode_to_charset(fribidiCharset, Visual, RtlLen, Rtl);
+  if (!cCharSetConv::SystemCharacterTable()) { // bidi requires UTF-8
+     fribidi_set_mirroring(true);
+     fribidi_set_reorder_nsm(false);
+     FriBidiCharSet fribidiCharset = FRIBIDI_CHAR_SET_UTF8;
+     int LtrLen = strlen(Ltr);
+     FriBidiCharType Base = FRIBIDI_TYPE_L;
+     FriBidiChar *Logical = MALLOC(FriBidiChar, LtrLen + 1) ;
+     int RtlLen = fribidi_charset_to_unicode(fribidiCharset, const_cast<char *>(Ltr), LtrLen, Logical);
+     FriBidiChar *Visual = MALLOC(FriBidiChar, LtrLen + 1) ;
+     char *Rtl = NULL;
+     bool ok = fribidi_log2vis(Logical, RtlLen, &Base, Visual, NULL, NULL, NULL);
+     if (ok) {
+        fribidi_remove_bidi_marks(Visual, RtlLen, NULL, NULL, NULL);
+        Rtl = MALLOC(char, RtlLen * 4 + 1);
+        fribidi_unicode_to_charset(fribidiCharset, Visual, RtlLen, Rtl);
+        }
+     free(Logical);
+     free(Visual);
+     if (ok)
+        return cString(Rtl, true);
      }
-  free(Logical);
-  free(Visual);
-  if (ok)
-     return cString(Rtl, true);
   return cString(Ltr);
 }
 #endif
@@ -589,7 +601,7 @@ void cTextWrapper::Set(const char *Text, const cFont *Font, int Width)
             p = Blank;
             continue;
             }
-         else {
+         else if (w > 0) { // there has to be at least one character before the newline
             // Here's the ugly part, where we don't have any whitespace to
             // punch in a newline, so we need to make room for it:
             if (Delim)
@@ -605,9 +617,8 @@ void cTextWrapper::Set(const char *Text, const cFont *Font, int Width)
             continue;
             }
          }
-      else
-         w += cw;
-      if (strchr("-.,:;!?_", *p)) {
+      w += cw;
+      if (strchr("-.,:;!?_~", *p)) {
          Delim = p;
          Blank = NULL;
          }
