@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: si.h 1.17 2007/04/22 13:32:09 kls Exp $
+ *   $Id: si.h 5.2 2025/03/02 11:03:35 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -28,7 +28,7 @@ enum TableId { TableIdPAT = 0x00, //program association section
                TableIdNIT_other  = 0x41, //network information section, other network
                TableIdSDT = 0x42, //service description section
                TableIdSDT_other  = 0x46,
-               TableIdBAT = 0x46, //bouquet association section
+               TableIdBAT = 0x4A, //bouquet association section
                TableIdEIT_presentFollowing = 0x4E, //event information section
                TableIdEIT_presentFollowing_other = 0x4F,
                //range from 0x50 to 0x5F
@@ -47,7 +47,6 @@ enum TableId { TableIdPAT = 0x00, //program association section
                TableIdPremiereCIT = 0xA0 //premiere content information section
              };
 
-
 enum DescriptorTag {
   // defined by ISO/IEC 13818-1
                VideoStreamDescriptorTag = 0x02,
@@ -70,6 +69,10 @@ enum DescriptorTag {
   // defined by ISO-13818-6 (DSM-CC)
                CarouselIdentifierDescriptorTag = 0x13,
                // 0x14 - 0x3F  Reserved
+  // defined by ISO/IEC 13818-1 Amendment
+               AVCDescriptorTag = 0x28,
+               SVCExtensionDescriptorTag = 0x30,
+               MVCExtensionDescriptorTag = 0x31,
   // defined by ETSI (EN 300 468)
                NetworkNameDescriptorTag = 0x40,
                ServiceListDescriptorTag = 0x41,
@@ -134,6 +137,31 @@ enum DescriptorTag {
                DTSDescriptorTag = 0x7B,
                AACDescriptorTag = 0x7C,
                ExtensionDescriptorTag = 0x7F,
+ // defined by EICTA/EACEM/DIGITALEUROPE
+               LogicalChannelDescriptorTag = 0x83,
+               PreferredNameListDescriptorTag = 0x84,
+               PreferredNameIdentifierDescriptorTag = 0x85,
+               EacemStreamIdentifierDescriptorTag = 0x86,
+               HdSimulcastLogicalChannelDescriptorTag = 0x88,
+ // Extension descriptors
+               ImageIconDescriptorTag = 0x00,
+               CpcmDeliverySignallingDescriptor = 0x01,
+               CPDescriptorTag = 0x02,
+               CPIdentifierDescriptorTag = 0x03,
+               T2DeliverySystemDescriptorTag = 0x04,
+               SHDeliverySystemDescriptorTag = 0x05,
+               SupplementaryAudioDescriptorTag = 0x06,
+               NetworkChangeNotifyDescriptorTag = 0x07,
+               MessageDescriptorTag = 0x08,
+               TargetRegionDescriptorTag = 0x09,
+               TargetRegionNameDescriptorTag = 0x0A,
+               ServiceRelocatedDescriptorTag = 0x0B,
+ // defined by ETSI (EN 300 468) v 1.12.1
+               XAITPidDescriptorTag = 0x0C,
+               C2DeliverySystemDescriptorTag = 0x0D,
+               // 0x0E - 0x0F Reserved
+               VideoDepthRangeDescriptorTag = 0x10,
+               T2MIDescriptorTag = 0x11,
 
  // Defined by ETSI TS 102 812 (MHP)
                // They once again start with 0x00 (see page 234, MHP specification)
@@ -153,6 +181,8 @@ enum DescriptorTag {
                MHP_PrefetchDescriptorTag = 0x0C,
                MHP_DelegatedApplicationDescriptorTag = 0x0E,
                MHP_ApplicationStorageDescriptorTag = 0x10,
+               MHP_SimpleApplicationLocationDescriptorTag = 0x15,
+               MHP_SimpleApplicationBoundaryDescriptorTag = 0x17,
   // Premiere private Descriptor Tags
                PremiereContentTransmissionDescriptorTag = 0xF2,
 
@@ -179,7 +209,8 @@ enum LinkageType { LinkageTypeInformationService = 0x01,
                    LinkageTypeRCSMap = 0x07,
                    LinkageTypeMobileHandover = 0x08,
                    LinkageTypeSystemSoftwareUpdateService = 0x09,
-                   LinkageTypeTSContainingSsuBatOrNit = 0x0A
+                   LinkageTypeTSContainingSsuBatOrNit = 0x0A,
+                   LinkageTypePremiere = 0xB0
                  };
 
 enum AudioType { AudioTypeUndefined = 0x00,
@@ -221,7 +252,7 @@ public:
    Section(const unsigned char *data, bool doCopy=true);
    Section() {}
    TableId getTableId() const;
-   virtual int getLength();
+   virtual int getLength() override;
 
    static int getLength(const unsigned char *d);
    static TableId getTableId(const unsigned char *d);
@@ -259,7 +290,7 @@ public:
    void setData(CharArray d, int l) { Object::setData(d); checkSize(l); length=l; }
    //convenience method
    void setDataAndOffset(CharArray d, int l, int &offset) { Object::setData(d); checkSize(l); length=l; offset+=l; }
-   virtual int getLength() { return length; }
+   virtual int getLength() override { return length; }
 private:
    int length;
 };
@@ -269,7 +300,7 @@ class LoopElement : public Object {
 
 class Descriptor : public LoopElement {
 public:
-   virtual int getLength();
+   virtual int getLength() override;
    DescriptorTag getDescriptorTag() const;
 
    static int getLength(const unsigned char *d);
@@ -300,7 +331,7 @@ public:
       int i;
    };
 protected:
-   virtual void Parse() {}
+   virtual void Parse() override {}
 };
 
 //contains LoopElements of one type only
@@ -332,8 +363,10 @@ public:
          T *ret=new T();
          ret->setData(d);
          ret->CheckParse();
-         if (!checkSize(ret->getLength()))
+         if (!checkSize(ret->getLength())) {
+            delete ret;
             return 0;
+         }
          it.i+=ret->getLength();
          return ret;
       }
@@ -411,6 +444,8 @@ public:
             return data.FourBytes(index);
          case 8:
             return (SixtyFourBit(data.FourBytes(index)) << 32) | data.FourBytes(index+4);
+         default:
+            return 0; // just to avoid a compiler warning
          }
          return 0; // just to avoid a compiler warning
       }
@@ -447,7 +482,7 @@ class DescriptorGroup {
 public:
    DescriptorGroup(bool deleteOnDesctruction=true);
    ~DescriptorGroup();
-   void Add(GroupDescriptor *d);
+   bool Add(GroupDescriptor *d);
    void Delete();
    int getLength() { return length; }
    GroupDescriptor **getDescriptors() { return array; }
@@ -472,7 +507,10 @@ public:
    //so the maximum there is 256.
    //returns the given buffer for convenience.
    //The emphasis marks 0x86 and 0x87 are still available.
-   char *getText(char *buffer, int size);
+   //If fromCode is given, the string will be copied into buffer in its raw form,
+   //without conversion, and he code table of the string is returned in this variable
+   //if it is NULL.
+   char *getText(char *buffer, int size, const char **fromCode = NULL);
    //The same semantics as for getText(char*) apply.
    //The short version of the text according to ETSI TR 101 211 (chapter 4.6)
    //will be written into the shortVersion buffer (which should, therefore, have the same
@@ -481,15 +519,30 @@ public:
    //The emphasis marks 0x86 and 0x87 are still available in buffer, but not in shortVersion.
    char *getText(char *buffer, char *shortVersion, int sizeBuffer, int sizeShortVersion);
 protected:
-   virtual void Parse() {}
-   void decodeText(char *buffer, int size);
+   virtual void Parse() override {}
+   void decodeText(char *buffer, int size, const char **fromCode = NULL);
    void decodeText(char *buffer, char *shortVersion, int sizeBuffer, int sizeShortVersion);
 };
 
+// Set the character table to use for strings that do not begin with a character
+// table indicator. Call with NULL to turn this off.
+// Must be called *after* SetSystemCharacterTable()!
+// Returns true if the character table was recognized.
+bool SetOverrideCharacterTable(const char *CharacterTable);
 // Call this function to set the system character table. CharacterTable is a string
 // like "iso8859-15" or "utf-8" (case insensitive).
 // Returns true if the character table was recognized.
 bool SetSystemCharacterTable(const char *CharacterTable);
+// Determines the character table used in the given buffer and returns
+// a string indicating that table. If no table can be determined, the
+// default ISO6937 is returned. If a table can be determined, the buffer
+// and length are adjusted accordingly.
+// The isSingleByte parameter is deprecated and only present for backwards compatibility.
+const char *getCharacterTable(const unsigned char *&buffer, int &length, bool *isSingleByte = NULL);
+// Copies 'from' to 'to' and converts characters according to 'fromCode', if given.
+// Returns the length of the resulting string.
+size_t convertCharacterTable(const char *from, size_t fromLength, char *to, size_t toLength, const char *fromCode);
+bool systemCharacterTableIsSingleByte(void);
 
 } //end of namespace
 
