@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.c 2.8 2010/12/12 13:41:09 kls Exp $
+ * $Id: menuitems.c 5.4 2025/06/18 20:22:10 kls Exp $
  */
 
 #include "menuitems.h"
@@ -26,6 +26,7 @@ const char *FileNameChars = trNOOP("FileNameChars$ abcdefghijklmnopqrstuvwxyz012
 cMenuEditItem::cMenuEditItem(const char *Name)
 {
   name = strdup(Name ? Name : "???");
+  SetHelp(NULL);
 }
 
 cMenuEditItem::~cMenuEditItem()
@@ -37,7 +38,27 @@ void cMenuEditItem::SetValue(const char *Value)
 {
   cString buffer = cString::sprintf("%s:\t%s", name, Value);
   SetText(buffer);
-  cStatus::MsgOsdCurrentItem(buffer);
+}
+
+void cMenuEditItem::SetHelp(const char *Red, const char *Green, const char *Yellow, const char *Blue)
+{
+  // strings are NOT copied - must be constants!!!
+  helpRed    = Red;
+  helpGreen  = Green;
+  helpYellow = Yellow;
+  helpBlue   = Blue;
+  helpDisplayed = false;
+}
+
+bool cMenuEditItem::DisplayHelp(bool Current)
+{
+  bool HasHelp = helpRed || helpGreen || helpYellow || helpBlue;
+  if (HasHelp && !helpDisplayed && Current) {
+     cSkinDisplay::Current()->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
+     cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+     }
+  helpDisplayed = Current;
+  return HasHelp;
 }
 
 // --- cMenuEditIntItem ------------------------------------------------------
@@ -79,7 +100,7 @@ eOSState cMenuEditIntItem::ProcessKey(eKeys Key)
      bool IsRepeat = Key & k_Repeat;
      Key = NORMALKEY(Key);
      switch (Key) {
-       case kNone: break;
+       case kNone: return state;
        case k0 ... k9:
             if (fresh) {
                newValue = 0;
@@ -201,6 +222,50 @@ eOSState cMenuEditNumItem::ProcessKey(eKeys Key)
   return state;
 }
 
+// --- cMenuEditIntxItem -----------------------------------------------------
+
+cMenuEditIntxItem::cMenuEditIntxItem(const char *Name, int *Value, int Min, int Max, int Factor, const char *NegString, const char *PosString)
+:cMenuEditIntItem(Name, Value, Min, Max)
+{
+  factor = ::max(Factor, 1);
+  negString = NegString;
+  posString = PosString;
+  Set();
+}
+
+void cMenuEditIntxItem::SetHelpKeys(void)
+{
+  if (negString && posString)
+     SetHelp(NULL, (*value < 0) ? posString : negString);
+}
+
+void cMenuEditIntxItem::Set(void)
+{
+  const char *s = (*value < 0) ? negString : posString;
+  int v = *value;
+  if (negString && posString)
+     v = abs(v);
+  SetValue(cString::sprintf(s ? "%.*f %s" : "%.*f", factor / 10, double(v) / factor, s));
+  SetHelpKeys();
+}
+
+eOSState cMenuEditIntxItem::ProcessKey(eKeys Key)
+{
+  eOSState state = cMenuEditIntItem::ProcessKey(Key);
+  if (state == osUnknown) {
+     switch (Key) {
+       case kGreen: if (negString && posString) {
+                       *value = -*value;
+                       Set();
+                       state = osContinue;
+                       }
+                    break;
+       default: ;
+       }
+     }
+  return state;
+}
+
 // --- cMenuEditPrcItem ------------------------------------------------------
 
 cMenuEditPrcItem::cMenuEditPrcItem(const char *Name, double *Value, double Min, double Max, int Decimals)
@@ -235,7 +300,7 @@ eOSState cMenuEditPrcItem::ProcessKey(eKeys Key)
      double newValue = round(*value * factor); // avoids precision problems
      Key = NORMALKEY(Key);
      switch (Key) {
-       case kNone: break;
+       case kNone: return state;
        case k0 ... k9:
             if (fresh) {
                newValue = 0;
@@ -324,6 +389,10 @@ cMenuEditStrItem::cMenuEditStrItem(const char *Name, char *Value, int Length, co
   allowed = Allowed ? Allowed : tr(FileNameChars);
   pos = -1;
   offset = 0;
+  keepSpace = false;
+  macro = -1;
+  lastMacro = -1;
+  macros = NULL;
   insert = uppercase = false;
   newchar = true;
   lengthUtf8 = 0;
@@ -342,6 +411,13 @@ cMenuEditStrItem::~cMenuEditStrItem()
   delete[] charMapUtf8;
 }
 
+void cMenuEditStrItem::SetMacros(const char **Macros)
+{
+  macros = Macros;
+  macro = 0;
+  lastMacro = -1;
+}
+
 void cMenuEditStrItem::EnterEditMode(void)
 {
   if (!valueUtf8) {
@@ -364,7 +440,8 @@ void cMenuEditStrItem::LeaveEditMode(bool SaveValue)
   if (valueUtf8) {
      if (SaveValue) {
         Utf8FromArray(valueUtf8, value, length);
-        stripspace(value);
+        if (!keepSpace)
+           stripspace(value);
         }
      lengthUtf8 = 0;
      delete[] valueUtf8;
@@ -382,9 +459,9 @@ void cMenuEditStrItem::LeaveEditMode(bool SaveValue)
 void cMenuEditStrItem::SetHelpKeys(void)
 {
   if (InEditMode())
-     cSkinDisplay::Current()->SetButtons(tr("Button$ABC/abc"), insert ? tr("Button$Overwrite") : tr("Button$Insert"), tr("Button$Delete"));
+     SetHelp(tr("Button$ABC/abc"), insert ? tr("Button$Overwrite") : tr("Button$Insert"), tr("Button$Delete"), macros ? tr("Button$Macro") : NULL);
   else
-     cSkinDisplay::Current()->SetButtons(NULL);
+     SetHelp(NULL);
 }
 
 uint *cMenuEditStrItem::IsAllowed(uint c)
@@ -419,14 +496,11 @@ void cMenuEditStrItem::AdvancePos(void)
 void cMenuEditStrItem::Set(void)
 {
   if (InEditMode()) {
-     // This is an ugly hack to make editing strings work with the 'skincurses' plugin.
      const cFont *font = dynamic_cast<cSkinDisplayMenu *>(cSkinDisplay::Current())->GetTextAreaFont(false);
-     if (!font || font->Width("W") != 1) // all characters have with == 1 in the font used by 'skincurses'
-        font = cFont::GetFont(fontOsd);
 
      int width = cSkinDisplay::Current()->EditableWidth();
      width -= font->Width("[]");
-     width -= font->Width("<>"); // reserving this anyway make the whole thing simpler
+     width -= font->Width("<>"); // reserving this anyway makes the whole thing simpler
 
      if (pos < offset)
         offset = pos;
@@ -515,11 +589,39 @@ void cMenuEditStrItem::Delete(void)
   lengthUtf8--;
 }
 
+void cMenuEditStrItem::InsertMacro(void)
+{
+  if (!macros)
+     return;
+  if (lastMacro >= 0) {
+     int l = strlen(macros[lastMacro]);
+     while (l-- > 0)
+           Delete();
+     }
+  const char *p = macros[macro];
+  int oldPos = pos;
+  bool oldInsert = insert;
+  insert = true;
+  newchar = true;
+  while (*p) {
+        Type(*p);
+        p++;
+        }
+  insert = oldInsert;
+  pos = oldPos;
+  lastMacro = macro;
+  if (!macros[++macro])
+     macro = 0;
+}
+
 eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
 {
   bool SameKey = NORMALKEY(Key) == lastKey;
-  if (Key != kNone)
+  if (Key != kNone) {
      lastKey = NORMALKEY(Key);
+     if (Key != kBlue)
+        lastMacro = -1;
+     }
   else if (!newchar && k0 <= lastKey && lastKey <= k9 && autoAdvanceTimeout.TimedOut()) {
      AdvancePos();
      newchar = true;
@@ -569,8 +671,9 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                     return osUnknown;
                  break;
     case kBlue|k_Repeat:
-    case kBlue:  // consume the key only if in edit-mode
-                 if (!InEditMode())
+    case kBlue:  if (InEditMode())
+                    InsertMacro();
+                 else
                     return osUnknown;
                  break;
     case kLeft|k_Repeat:
@@ -708,12 +811,45 @@ void cMenuEditStraItem::Set(void)
   SetValue(strings[*value]);
 }
 
+// --- cMenuEditStrlItem -----------------------------------------------------
+
+cMenuEditStrlItem::cMenuEditStrlItem(const char *Name, char *Value, int Length, const cStringList *Strings)
+:cMenuEditIntItem(Name, &index, 0, Strings->Size() - 1)
+{
+  strings = Strings;
+  value = Value;
+  length = Length;
+  index = strings->Find(value);
+  if (index < 0)
+     index = 0;
+  Set();
+}
+
+void cMenuEditStrlItem::Set(void)
+{
+  strn0cpy(value, strings->At(index), length);
+  SetValue(value);
+}
+
 // --- cMenuEditChanItem -----------------------------------------------------
 
 cMenuEditChanItem::cMenuEditChanItem(const char *Name, int *Value, const char *NoneString)
-:cMenuEditIntItem(Name, Value, NoneString ? 0 : 1, Channels.MaxNumber())
+:cMenuEditIntItem(Name, Value, NoneString ? 0 : 1, cChannels::MaxNumber())
 {
+  channelID = NULL;
   noneString = NoneString;
+  dummyValue = 0;
+  Set();
+}
+
+cMenuEditChanItem::cMenuEditChanItem(const char *Name, cString *ChannelID, const char *NoneString)
+:cMenuEditIntItem(Name, &dummyValue, NoneString ? 0 : 1, cChannels::MaxNumber())
+{
+  channelID = ChannelID;
+  noneString = NoneString;
+  LOCK_CHANNELS_READ;
+  const cChannel *Channel = Channels->GetByChannelID(tChannelID::FromString(*ChannelID));
+  dummyValue = Channel ? Channel->Number() : 0;
   Set();
 }
 
@@ -721,12 +857,18 @@ void cMenuEditChanItem::Set(void)
 {
   if (*value > 0) {
      char buf[255];
-     cChannel *channel = Channels.GetByNumber(*value);
-     snprintf(buf, sizeof(buf), "%d %s", *value, channel ? channel->Name() : "");
+     LOCK_CHANNELS_READ;
+     const cChannel *Channel = Channels->GetByNumber(*value);
+     snprintf(buf, sizeof(buf), "%d %s", *value, Channel ? Channel->Name() : "");
      SetValue(buf);
+     if (channelID)
+        *channelID = Channel ? Channel->GetChannelID().ToString() : "";
      }
-  else if (noneString)
+  else if (noneString) {
      SetValue(noneString);
+     if (channelID)
+        *channelID = "";
+     }
 }
 
 eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
@@ -739,11 +881,14 @@ eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
     case kRight|k_Repeat:
     case kRight:
                  {
-                   cChannel *channel = Channels.GetByNumber(*value + delta, delta);
-                   if (channel)
-                      *value = channel->Number();
+                   LOCK_CHANNELS_READ
+                   const cChannel *Channel = Channels->GetByNumber(*value + delta, delta);
+                   if (Channel)
+                      *value = Channel->Number();
                    else if (delta < 0 && noneString)
                       *value = 0;
+                   if (channelID)
+                      *channelID = Channel ? Channel->GetChannelID().ToString() : "";
                    Set();
                  }
                  break;
@@ -760,13 +905,14 @@ cMenuEditTranItem::cMenuEditTranItem(const char *Name, int *Value, int *Source)
   number = 0;
   source = Source;
   transponder = Value;
-  cChannel *channel = Channels.First();
-  while (channel) {
-        if (!channel->GroupSep() && *source == channel->Source() && ISTRANSPONDER(channel->Transponder(), *Value)) {
-           number = channel->Number();
+  LOCK_CHANNELS_READ;
+  const cChannel *Channel = Channels->First();
+  while (Channel) {
+        if (!Channel->GroupSep() && *source == Channel->Source() && ISTRANSPONDER(Channel->Transponder(), *Value)) {
+           number = Channel->Number();
            break;
            }
-        channel = (cChannel *)channel->Next();
+        Channel = Channels->Next(Channel);
         }
   Set();
 }
@@ -774,10 +920,10 @@ cMenuEditTranItem::cMenuEditTranItem(const char *Name, int *Value, int *Source)
 eOSState cMenuEditTranItem::ProcessKey(eKeys Key)
 {
   eOSState state = cMenuEditChanItem::ProcessKey(Key);
-  cChannel *channel = Channels.GetByNumber(number);
-  if (channel) {
-     *source = channel->Source();
-     *transponder = channel->Transponder();
+  LOCK_CHANNELS_READ
+  if (const cChannel *Channel = Channels->GetByNumber(number)) {
+     *source = Channel->Source();
+     *transponder = Channel->Transponder();
      }
   else {
      *source = 0;
@@ -814,6 +960,7 @@ cMenuEditDateItem::cMenuEditDateItem(const char *Name, time_t *Value, int *WeekD
   value = Value;
   weekdays = WeekDays;
   oldvalue = 0;
+  oldweekdays = 0;
   dayindex = weekdays ? FindDayIndex(*weekdays) : 0;
   Set();
 }
@@ -845,6 +992,26 @@ void cMenuEditDateItem::Set(void)
   SetValue(buf);
 }
 
+void cMenuEditDateItem::ToggleRepeating(void)
+{
+  if (weekdays) {
+     if (*weekdays) {
+        *value = cTimer::SetTime(oldvalue ? oldvalue : time(NULL), 0);
+        oldvalue = 0;
+        oldweekdays = *weekdays;
+        *weekdays = 0;
+        }
+     else {
+        *weekdays = oldweekdays ? oldweekdays : days[cTimer::GetWDay(*value)];
+        oldweekdays = 0;
+        dayindex = FindDayIndex(*weekdays);
+        oldvalue = *value;
+        *value = 0;
+        }
+     Set();
+     }
+}
+
 eOSState cMenuEditDateItem::ProcessKey(eKeys Key)
 {
   eOSState state = cMenuEditItem::ProcessKey(Key);
@@ -896,17 +1063,8 @@ eOSState cMenuEditDateItem::ProcessKey(eKeys Key)
      else if (weekdays) {
         if (Key == k0) {
            // Toggle between weekdays and single day:
-           if (*weekdays) {
-              *value = cTimer::SetTime(oldvalue ? oldvalue : now, 0);
-              oldvalue = 0;
-              *weekdays = 0;
-              }
-           else {
-              *weekdays = days[cTimer::GetWDay(*value)];
-              dayindex = FindDayIndex(*weekdays);
-              oldvalue = *value;
-              *value = 0;
-              }
+           ToggleRepeating();
+           return osContinue; // ToggleRepeating) has already called Set()
            }
         else if (k1 <= Key && Key <= k7) {
            // Toggle individual weekdays:
@@ -941,14 +1099,12 @@ cMenuEditTimeItem::cMenuEditTimeItem(const char *Name, int *Value)
 
 void cMenuEditTimeItem::Set(void)
 {
-  char buf[10];
   switch (pos) {
-    case 1:  snprintf(buf, sizeof(buf), "%01d-:--", hh / 10); break;
-    case 2:  snprintf(buf, sizeof(buf), "%02d:--", hh); break;
-    case 3:  snprintf(buf, sizeof(buf), "%02d:%01d-", hh, mm / 10); break;
-    default: snprintf(buf, sizeof(buf), "%02d:%02d", hh, mm);
+    case 1:  SetValue(cString::sprintf("%01d-:--", hh / 10)); break;
+    case 2:  SetValue(cString::sprintf("%02d:--", hh)); break;
+    case 3:  SetValue(cString::sprintf("%02d:%01d-", hh, mm / 10)); break;
+    default: SetValue(cString::sprintf("%02d:%02d", hh, mm));
     }
-  SetValue(buf);
 }
 
 eOSState cMenuEditTimeItem::ProcessKey(eKeys Key)
@@ -1071,8 +1227,9 @@ eOSState cMenuEditMapItem::ProcessKey(eKeys Key)
 // --- cMenuSetupPage --------------------------------------------------------
 
 cMenuSetupPage::cMenuSetupPage(void)
-:cOsdMenu("", 33)
+:cOsdMenu("", 36)
 {
+  SetMenuCategory(mcSetup);
   plugin = NULL;
 }
 
@@ -1098,6 +1255,7 @@ eOSState cMenuSetupPage::ProcessKey(eKeys Key)
 
 void cMenuSetupPage::SetPlugin(cPlugin *Plugin)
 {
+  SetMenuCategory(mcPluginSetup);
   plugin = Plugin;
   SetSection(cString::sprintf("%s '%s'", tr("Plugin"), plugin->Name()));
 }
