@@ -4,13 +4,14 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.c 2.3 2010/12/12 13:41:28 kls Exp $
+ * $Id: osdbase.c 5.10 2025/06/18 20:22:10 kls Exp $
  */
 
 #include "osdbase.h"
 #include <string.h>
 #include "device.h"
 #include "i18n.h"
+#include "menuitems.h"
 #include "remote.h"
 #include "status.h"
 
@@ -54,6 +55,11 @@ void cOsdItem::SetFresh(bool Fresh)
   fresh = Fresh;
 }
 
+void cOsdItem::SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable)
+{
+  DisplayMenu->SetItem(Text(), Index, Current, Selectable);
+}
+
 eOSState cOsdItem::ProcessKey(eKeys Key)
 {
   return Key == kOk ? state : osUnknown;
@@ -71,23 +77,36 @@ void cOsdObject::Show(void)
 
 cSkinDisplayMenu *cOsdMenu::displayMenu = NULL;
 int cOsdMenu::displayMenuCount = 0;
-int cOsdMenu::displayMenuItems = 0;//XXX dynamic???
+int cOsdMenu::osdState = 0;
+cOsdMenu *cOsdMenu::topMenu = NULL;
 
 cOsdMenu::cOsdMenu(const char *Title, int c0, int c1, int c2, int c3, int c4)
 {
   isMenu = true;
   digit = 0;
   hasHotkeys = false;
+  if (!topMenu)
+     topMenu = this;
+  active = this == topMenu;
+  displayMenuItems = 0;
   title = NULL;
+  menuCategory = mcUnknown;
+  menuSortMode = msmUnknown;
+  menuOrientation = moVertical;
   SetTitle(Title);
   SetCols(c0, c1, c2, c3, c4);
   first = 0;
+  lastOffset = -1;
+  conveyStatus = true;
   current = marked = -1;
   subMenu = NULL;
   helpRed = helpGreen = helpYellow = helpBlue = NULL;
+  helpDisplayed = false;
   status = NULL;
-  if (!displayMenuCount++)
+  if (!displayMenuCount++) {
+     cOsdProvider::OsdSizeChanged(osdState); // to get the current state
      SetDisplayMenu();
+     }
 }
 
 cOsdMenu::~cOsdMenu()
@@ -99,6 +118,23 @@ cOsdMenu::~cOsdMenu()
   cStatus::MsgOsdClear();
   if (!--displayMenuCount)
      DELETENULL(displayMenu);
+  if (this == topMenu)
+     topMenu = NULL;
+}
+
+void cOsdMenu::SetMenuCategory(eMenuCategory MenuCategory)
+{
+  menuCategory = MenuCategory;
+}
+
+void cOsdMenu::SetMenuSortMode(eMenuSortMode MenuSortMode)
+{
+  menuSortMode = MenuSortMode;
+}
+
+void cOsdMenu::SetActive(bool Active)
+{
+  active = Active;
 }
 
 void cOsdMenu::SetDisplayMenu(void)
@@ -108,7 +144,6 @@ void cOsdMenu::SetDisplayMenu(void)
      delete displayMenu;
      }
   displayMenu = Skins.Current()->DisplayMenu();
-  displayMenuItems = displayMenu->MaxItems();
 }
 
 const char *cOsdMenu::hk(const char *s)
@@ -146,6 +181,7 @@ void cOsdMenu::SetStatus(const char *s)
   free(status);
   status = s ? strdup(s) : NULL;
   displayMenu->SetMessage(mtStatus, s);
+  cStatus::MsgOsdStatusMessage(mtStatus, s);
 }
 
 void cOsdMenu::SetTitle(const char *Title)
@@ -154,6 +190,16 @@ void cOsdMenu::SetTitle(const char *Title)
   title = strdup(Title);
 }
 
+void cOsdMenu::DisplayHelp(bool Force)
+{
+  if (!helpDisplayed || Force) {
+     displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
+     if (conveyStatus)
+        cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+     helpDisplayed = true;
+     }
+}
+
 void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, const char *Blue)
 {
   // strings are NOT copied - must be constants!!!
@@ -161,8 +207,7 @@ void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, c
   helpGreen  = Green;
   helpYellow = Yellow;
   helpBlue   = Blue;
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  DisplayHelp(true);
 }
 
 void cOsdMenu::Del(int Index)
@@ -193,31 +238,50 @@ void cOsdMenu::Ins(cOsdItem *Item, bool Current, cOsdItem *Before)
      current = Item->Index();
 }
 
+void cOsdMenu::DisplayNoStatus(void)
+{
+  conveyStatus = false;
+  Display();
+  conveyStatus = true;
+}
+
 void cOsdMenu::Display(void)
 {
   if (subMenu) {
      subMenu->Display();
      return;
      }
+  if (!active)
+     return;
+  if (cOsdProvider::OsdSizeChanged(osdState))
+     SetDisplayMenu();
   displayMenu->SetMessage(mtStatus, NULL);
+  cStatus::MsgOsdStatusMessage(mtStatus, NULL);
   displayMenu->Clear();
-  cStatus::MsgOsdClear();
+  if (conveyStatus)
+     cStatus::MsgOsdClear();
+  if (menuCategory != displayMenu->MenuCategory())
+     displayMenu->SetMenuCategory(menuCategory);
+  displayMenu->SetMenuSortMode(menuSortMode);
+  menuOrientation = displayMenu->MenuOrientation();
+  displayMenuItems = displayMenu->MaxItems();
   displayMenu->SetTabs(cols[0], cols[1], cols[2], cols[3], cols[4]);//XXX
   displayMenu->SetTitle(title);
-  cStatus::MsgOsdTitle(title);
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  if (conveyStatus)
+     cStatus::MsgOsdTitle(title);
+  DisplayHelp(true);
   int count = Count();
   if (count > 0) {
      int ni = 0;
      for (cOsdItem *item = First(); item; item = Next(item)) {
-         cStatus::MsgOsdItem(item->Text(), ni++);
+         if (conveyStatus)
+            cStatus::MsgOsdItem(item->Text(), ni++, item->Selectable());
          if (current < 0 && item->Selectable())
             current = item->Index();
          }
      if (current < 0)
         current = 0; // just for safety - there HAS to be a current item!
-     first = min(first, max(0, count - displayMenuItems)); // in case the menu size has changed
+     first = max(0, min(first, max(0, count - displayMenuItems))); // in case the menu size has changed
      if (current - first >= displayMenuItems || current < first) {
         first = current - displayMenuItems / 2;
         if (first + displayMenuItems > count)
@@ -229,22 +293,27 @@ void cOsdMenu::Display(void)
      int n = 0;
      for (cOsdItem *item = Get(first); item; item = Next(item)) {
          bool CurrentSelectable = (i == current) && item->Selectable();
-         displayMenu->SetItem(item->Text(), i - first, CurrentSelectable, item->Selectable());
-         if (CurrentSelectable)
-            cStatus::MsgOsdCurrentItem(item->Text());
+         item->SetMenuItem(displayMenu, i - first, CurrentSelectable, item->Selectable());
+         if (CurrentSelectable) // not checking conveyStatus here!
+            cStatus::MsgOsdCurrentItem(item->Text(), i);
          if (++n == displayMenuItems)
             break;
          i++;
          }
      }
   displayMenu->SetScrollbar(count, first);
-  if (!isempty(status))
+  if (!isempty(status)) {
      displayMenu->SetMessage(mtStatus, status);
+     cStatus::MsgOsdStatusMessage(mtStatus, status);
+     }
 }
 
 void cOsdMenu::SetCurrent(cOsdItem *Item)
 {
   current = Item ? Item->Index() : -1;
+  if (current >= 0 && lastOffset >= 0)
+     first = max(0, current - lastOffset);
+  lastOffset = -1;
 }
 
 void cOsdMenu::RefreshCurrent(void)
@@ -258,11 +327,21 @@ void cOsdMenu::DisplayCurrent(bool Current)
 {
   cOsdItem *item = Get(current);
   if (item) {
-     displayMenu->SetItem(item->Text(), current - first, Current && item->Selectable(), item->Selectable());
-     if (Current && item->Selectable())
-        cStatus::MsgOsdCurrentItem(item->Text());
-     if (!Current)
+     item->SetMenuItem(displayMenu, current - first, Current && item->Selectable(), item->Selectable());
+     if (Current) {
+        if (current - first >= displayMenuItems || current < first)
+           DisplayNoStatus();
+        else if (item->Selectable())
+           cStatus::MsgOsdCurrentItem(item->Text(), current);
+        }
+     else
         item->SetFresh(true); // leaving the current item resets 'fresh'
+     if (cMenuEditItem *MenuEditItem = dynamic_cast<cMenuEditItem *>(item)) {
+        if (!MenuEditItem->DisplayHelp(Current))
+           DisplayHelp();
+        else
+           helpDisplayed = false;
+        }
      }
 }
 
@@ -273,9 +352,9 @@ void cOsdMenu::DisplayItem(cOsdItem *Item)
      int Offset = Index - first;
      if (Offset >= 0 && Offset < first + displayMenuItems) {
         bool Current = Index == current;
-        displayMenu->SetItem(Item->Text(), Offset, Current && Item->Selectable(), Item->Selectable());
+        Item->SetMenuItem(displayMenu, Offset, Current && Item->Selectable(), Item->Selectable());
         if (Current && Item->Selectable())
-           cStatus::MsgOsdCurrentItem(Item->Text());
+           cStatus::MsgOsdCurrentItem(Item->Text(), Index);
         }
      }
 }
@@ -284,6 +363,8 @@ void cOsdMenu::Clear(void)
 {
   if (marked >= 0)
      SetStatus(NULL);
+  if (current >= 0)
+     lastOffset = (current > first) ? current - first : 0;
   first = 0;
   current = marked = -1;
   cList<cOsdItem>::Clear();
@@ -307,7 +388,7 @@ void cOsdMenu::CursorUp(void)
            if (first > 0) {
               // make non-selectable items at the beginning visible:
               first = 0;
-              Display();
+              DisplayNoStatus();
               return;
               }
            if (Setup.MenuScrollWrap)
@@ -323,11 +404,11 @@ void cOsdMenu::CursorUp(void)
   current = tmpCurrent;
   if (current < first) {
      first = Setup.MenuScrollPage ? max(0, current - displayMenuItems + 1) : current;
-     Display();
+     DisplayNoStatus();
      }
   else if (current > lastOnScreen) {
      first = max(0, current - displayMenuItems + 1);
-     Display();
+     DisplayNoStatus();
      }
   else
      DisplayCurrent(true);
@@ -345,7 +426,7 @@ void cOsdMenu::CursorDown(void)
            if (first < last - displayMenuItems) {
               // make non-selectable items at the end visible:
               first = last - displayMenuItems + 1;
-              Display();
+              DisplayNoStatus();
               return;
               }
            if (Setup.MenuScrollWrap)
@@ -363,11 +444,11 @@ void cOsdMenu::CursorDown(void)
      first = Setup.MenuScrollPage ? current : max(0, current - displayMenuItems + 1);
      if (first + displayMenuItems > last)
         first = max(0, last - displayMenuItems + 1);
-     Display();
+     DisplayNoStatus();
      }
   else if (current < first) {
      first = current;
-     Display();
+     DisplayNoStatus();
      }
   else
      DisplayCurrent(true);
@@ -399,10 +480,8 @@ void cOsdMenu::PageUp(void)
      else if (current - first >= displayMenuItems)
         first = current - displayMenuItems + 1;
      }
-  if (current != oldCurrent || first != oldFirst) {
-     Display();
-     DisplayCurrent(true);
-     }
+  if (current != oldCurrent || first != oldFirst)
+     DisplayNoStatus();
   else if (Setup.MenuScrollWrap)
      CursorUp();
 }
@@ -433,10 +512,8 @@ void cOsdMenu::PageDown(void)
      else if (current - first >= displayMenuItems)
         first = current - displayMenuItems + 1;
      }
-  if (current != oldCurrent || first != oldFirst) {
-     Display();
-     DisplayCurrent(true);
-     }
+  if (current != oldCurrent || first != oldFirst)
+     DisplayNoStatus();
   else if (Setup.MenuScrollWrap)
      CursorDown();
 }
@@ -455,9 +532,10 @@ eOSState cOsdMenu::HotKey(eKeys Key)
       const char *s = item->Text();
       if (s && (s = skipspace(s)) != NULL) {
          if (*s == Key - k1 + '1') {
+            DisplayCurrent(false);
             current = item->Index();
             RefreshCurrent();
-            Display();
+            DisplayCurrent(true);
             cRemote::Put(kOk, true);
             break;
             }
@@ -468,18 +546,23 @@ eOSState cOsdMenu::HotKey(eKeys Key)
 
 eOSState cOsdMenu::AddSubMenu(cOsdMenu *SubMenu)
 {
+  SetActive(false);
   delete subMenu;
   subMenu = SubMenu;
+  subMenu->SetActive(true);
   subMenu->Display();
   return osContinue; // convenience return value
 }
 
-eOSState cOsdMenu::CloseSubMenu()
+eOSState cOsdMenu::CloseSubMenu(bool ReDisplay)
 {
   delete subMenu;
   subMenu = NULL;
-  RefreshCurrent();
-  Display();
+  SetActive(true);
+  if (ReDisplay) {
+     RefreshCurrent();
+     Display();
+     }
   return osContinue; // convenience return value
 }
 
@@ -504,13 +587,13 @@ eOSState cOsdMenu::ProcessKey(eKeys Key)
     case k0:      return osUnknown;
     case k1...k9: return hasHotkeys ? HotKey(Key) : osUnknown;
     case kUp|k_Repeat:
-    case kUp:   CursorUp();   break;
+    case kUp:     if (menuOrientation == moHorizontal) PageUp();     else CursorUp(); break;
     case kDown|k_Repeat:
-    case kDown: CursorDown(); break;
+    case kDown:   if (menuOrientation == moHorizontal) PageDown();   else CursorDown(); break;
     case kLeft|k_Repeat:
-    case kLeft: PageUp(); break;
+    case kLeft:   if (menuOrientation == moHorizontal) CursorUp();   else PageUp(); break;
     case kRight|k_Repeat:
-    case kRight: PageDown(); break;
+    case kRight:  if (menuOrientation == moHorizontal) CursorDown(); else PageDown(); break;
     case kBack: return osBack;
     case kOk:   if (marked >= 0) {
                    SetStatus(NULL);
