@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.h 1.17 2007/11/03 14:50:52 kls Exp $
+ * $Id: osdbase.h 5.4 2026/01/10 20:26:08 kls Exp $
  */
 
 #ifndef __OSDBASE_H
@@ -21,6 +21,8 @@ enum eOSState { osUnknown,
                 osChannels,
                 osTimers,
                 osRecordings,
+                osRecsOpen,
+                osRecsDel,
                 osPlugin,
                 osSetup,
                 osCommands,
@@ -30,7 +32,6 @@ enum eOSState { osUnknown,
                 osStopRecord,
                 osStopReplay,
                 osCancelEdit,
-                osSwitchDvb,
                 osBack,
                 osEnd,
                 os_User, // the following values can be used locally
@@ -56,13 +57,14 @@ protected:
 public:
   cOsdItem(eOSState State = osUnknown);
   cOsdItem(const char *Text, eOSState State = osUnknown, bool Selectable = true);
-  virtual ~cOsdItem();
+  virtual ~cOsdItem() override;
   bool Selectable(void) const { return selectable; }
   void SetText(const char *Text, bool Copy = true);
   void SetSelectable(bool Selectable);
   void SetFresh(bool Fresh);
   const char *Text(void) const { return text; }
   virtual void Set(void) {}
+  virtual void SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable);
   virtual eOSState ProcessKey(eKeys Key);
   };
 
@@ -86,15 +88,27 @@ class cOsdMenu : public cOsdObject, public cList<cOsdItem> {
 private:
   static cSkinDisplayMenu *displayMenu;
   static int displayMenuCount;
-  static int displayMenuItems;
+  static int osdState;
+  static cOsdMenu *topMenu;
+  int displayMenuItems;
   char *title;
   int cols[cSkinDisplayMenu::MaxTabs];
   int first, current, marked;
+  int lastOffset;
+  bool conveyStatus;
+  eMenuCategory menuCategory;
+  eMenuSortMode menuSortMode;
+  eMenuOrientation menuOrientation;
   cOsdMenu *subMenu;
   const char *helpRed, *helpGreen, *helpYellow, *helpBlue;
+  bool helpDisplayed;
   char *status;
   int digit;
   bool hasHotkeys;
+  bool active;
+  void SetActive(bool Active);
+  void DisplayHelp(bool Force = false);
+  void DisplayNoStatus(void);
 protected:
   void SetDisplayMenu(void);
   cSkinDisplayMenu *DisplayMenu(void) { return displayMenu; }
@@ -102,6 +116,7 @@ protected:
   void SetCols(int c0, int c1 = 0, int c2 = 0, int c3 = 0, int c4 = 0);
   void SetHasHotkeys(bool HasHotkeys = true);
   virtual void Clear(void);
+  const char *Title(void) { return title; }
   bool SelectableItem(int idx);
   void SetCurrent(cOsdItem *Item);
   void RefreshCurrent(void);
@@ -114,21 +129,24 @@ protected:
   void Mark(void);
   eOSState HotKey(eKeys Key);
   eOSState AddSubMenu(cOsdMenu *SubMenu);
-  eOSState CloseSubMenu();
+  eOSState CloseSubMenu(bool ReDisplay = true);
   bool HasSubMenu(void) { return subMenu; }
+  cOsdMenu *SubMenu(void) { return subMenu; }
   void SetStatus(const char *s);
   void SetTitle(const char *Title);
   void SetHelp(const char *Red, const char *Green = NULL, const char *Yellow = NULL, const char *Blue = NULL);
   virtual void Del(int Index);
 public:
   cOsdMenu(const char *Title, int c0 = 0, int c1 = 0, int c2 = 0, int c3 = 0, int c4 = 0);
-  virtual ~cOsdMenu();
-  virtual bool NeedsFastResponse(void) { return subMenu ? subMenu->NeedsFastResponse() : cOsdObject::NeedsFastResponse(); }
+  virtual ~cOsdMenu() override;
+  virtual bool NeedsFastResponse(void) override { return subMenu ? subMenu->NeedsFastResponse() : cOsdObject::NeedsFastResponse(); }
+  void SetMenuCategory(eMenuCategory MenuCategory);
+  void SetMenuSortMode(eMenuSortMode MenuSortMode);
   int Current(void) const { return current; }
   void Add(cOsdItem *Item, bool Current = false, cOsdItem *After = NULL);
   void Ins(cOsdItem *Item, bool Current = false, cOsdItem *Before = NULL);
   virtual void Display(void);
-  virtual eOSState ProcessKey(eKeys Key);
+  virtual eOSState ProcessKey(eKeys Key) override;
   };
 
 #endif //__OSDBASE_H
