@@ -4,9 +4,10 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: plugin.c 1.28 2008/02/17 13:32:12 kls Exp $
+ * $Id: plugin.c 5.1 2025/02/12 22:22:20 kls Exp $
  */
 
+#define MUTE_DEPRECATED_MAINTHREADHOOK
 #include "plugin.h"
 #include <ctype.h>
 #include <dirent.h>
@@ -25,7 +26,9 @@
 
 // --- cPlugin ---------------------------------------------------------------
 
-char *cPlugin::configDirectory = NULL;
+cString cPlugin::configDirectory;
+cString cPlugin::cacheDirectory;
+cString cPlugin::resourceDirectory;
 
 cPlugin::cPlugin(void)
 {
@@ -130,14 +133,9 @@ cString cPlugin::SVDRPCommand(const char *Command, const char *Option, int &Repl
   return NULL;
 }
 
-void cPlugin::RegisterI18n(const void *)
-{
-  dsyslog("plugin '%s' called obsolete function RegisterI18n()", Name());
-}
-
 void cPlugin::SetConfigDirectory(const char *Dir)
 {
-  configDirectory = strdup(Dir);
+  configDirectory = Dir;
 }
 
 const char *cPlugin::ConfigDirectory(const char *PluginName)
@@ -145,7 +143,35 @@ const char *cPlugin::ConfigDirectory(const char *PluginName)
   static cString buffer;
   if (!cThread::IsMainThread())
      esyslog("ERROR: plugin '%s' called cPlugin::ConfigDirectory(), which is not thread safe!", PluginName ? PluginName : "<no name given>");
-  buffer = cString::sprintf("%s/plugins%s%s", configDirectory, PluginName ? "/" : "", PluginName ? PluginName : "");
+  buffer = cString::sprintf("%s/plugins%s%s", *configDirectory, PluginName ? "/" : "", PluginName ? PluginName : "");
+  return MakeDirs(buffer, true) ? *buffer : NULL;
+}
+
+void cPlugin::SetCacheDirectory(const char *Dir)
+{
+  cacheDirectory = Dir;
+}
+
+const char *cPlugin::CacheDirectory(const char *PluginName)
+{
+  static cString buffer;
+  if (!cThread::IsMainThread())
+     esyslog("ERROR: plugin '%s' called cPlugin::CacheDirectory(), which is not thread safe!", PluginName ? PluginName : "<no name given>");
+  buffer = cString::sprintf("%s/plugins%s%s", *cacheDirectory, PluginName ? "/" : "", PluginName ? PluginName : "");
+  return MakeDirs(buffer, true) ? *buffer : NULL;
+}
+
+void cPlugin::SetResourceDirectory(const char *Dir)
+{
+  resourceDirectory = Dir;
+}
+
+const char *cPlugin::ResourceDirectory(const char *PluginName)
+{
+  static cString buffer;
+  if (!cThread::IsMainThread())
+     esyslog("ERROR: plugin '%s' called cPlugin::ResourceDirectory(), which is not thread safe!", PluginName ? PluginName : "<no name given>");
+  buffer = cString::sprintf("%s/plugins%s%s", *resourceDirectory, PluginName ? "/" : "", PluginName ? PluginName : "");
   return MakeDirs(buffer, true) ? *buffer : NULL;
 }
 
@@ -157,11 +183,15 @@ cDll::cDll(const char *FileName, const char *Args)
   args = Args ? strdup(Args) : NULL;
   handle = NULL;
   plugin = NULL;
+  destroy = NULL;
 }
 
 cDll::~cDll()
 {
-  delete plugin;
+  if (destroy)
+     destroy(plugin);
+  else
+     delete plugin; // silently fall back for plugins compiled with VDR version <= 2.4.3
   if (handle)
      dlclose(handle);
   free(args);
@@ -171,15 +201,15 @@ cDll::~cDll()
 static char *SkipQuote(char *s)
 {
   char c = *s;
-  strcpy(s, s + 1);
+  memmove(s, s + 1, strlen(s));
   while (*s && *s != c) {
         if (*s == '\\')
-           strcpy(s, s + 1);
+           memmove(s, s + 1, strlen(s));
         if (*s)
            s++;
         }
   if (*s) {
-     strcpy(s, s + 1);
+     memmove(s, s + 1, strlen(s));
      return s;
      }
   esyslog("ERROR: missing closing %c", c);
@@ -198,10 +228,18 @@ bool cDll::Load(bool Log)
   handle = dlopen(fileName, RTLD_NOW);
   const char *error = dlerror();
   if (!error) {
-     void *(*creator)(void);
-     creator = (void *(*)(void))dlsym(handle, "VDRPluginCreator");
-     if (!(error = dlerror()))
-        plugin = (cPlugin *)creator();
+     typedef cPlugin *create_t(void);
+     create_t *create = (create_t *)dlsym(handle, "VDRPluginCreator");
+     error = dlerror();
+     if (!error && create) {
+        plugin = create();
+        destroy = (destroy_t *)dlsym(handle, "VDRPluginDestroyer");
+        error = dlerror();
+        if (error) {
+           error = NULL;
+           isyslog("plugin %s: missing symbol VDRPluginDestroyer(), please rebuild", fileName);
+           }
+        }
      }
   if (!error) {
      if (plugin && args) {
@@ -214,7 +252,7 @@ bool cDll::Load(bool Log)
               if (!q)
                  q = p;
               switch (*p) {
-                case '\\': strcpy(p, p + 1);
+                case '\\': memmove(p, p + 1, strlen(p));
                            if (*p)
                               p++;
                            else {
@@ -293,23 +331,23 @@ void cPluginManager::SetDirectory(const char *Directory)
 void cPluginManager::AddPlugin(const char *Args)
 {
   if (strcmp(Args, "*") == 0) {
-     cReadDir d(directory);
-     struct dirent *e;
-     while ((e = d.Next()) != NULL) {
-           if (strstr(e->d_name, LIBVDR_PREFIX) == e->d_name) {
-              char *p = strstr(e->d_name, SO_INDICATOR);
-              if (p) {
-                 *p = 0;
-                 p += strlen(SO_INDICATOR);
-                 if (strcmp(p, APIVERSION) == 0) {
-                    char *name = e->d_name + strlen(LIBVDR_PREFIX);
-                    if (strcmp(name, "*") != 0) { // let's not get into a loop!
-                       AddPlugin(e->d_name + strlen(LIBVDR_PREFIX));
-                       }
-                    }
-                 }
-              }
-           }
+     cFileNameList Files(directory);
+     for (int i = 0; i < Files.Size(); i++) {
+         char *FileName = Files.At(i);
+         if (strstr(FileName, LIBVDR_PREFIX) == FileName) {
+            char *p = strstr(FileName, SO_INDICATOR);
+            if (p) {
+               *p = 0;
+               p += strlen(SO_INDICATOR);
+               if (strcmp(p, APIVERSION) == 0) {
+                  char *name = FileName + strlen(LIBVDR_PREFIX);
+                  if (strcmp(name, "*") != 0) { // let's not get into a loop!
+                     AddPlugin(FileName + strlen(LIBVDR_PREFIX));
+                     }
+                  }
+               }
+            }
+         }
      return;
      }
   char *s = strdup(skipspace(Args));
