@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 5.15 2025/04/16 09:14:20 kls Exp $
+ * $Id: recording.h 5.19 2026/01/04 10:18:40 kls Exp $
  */
 
 #ifndef __RECORDING_H
@@ -79,6 +79,7 @@ private:
   int lifetime;
   char *fileName;
   int errors;
+  int tmpErrors;
   cRecordingInfo(const cChannel *Channel = NULL, const cEvent *Event = NULL);
   bool Read(FILE *f, bool Force = false);
 public:
@@ -108,7 +109,8 @@ public:
   void SetFrameParams(uint16_t FrameWidth, uint16_t FrameHeight, eScanType ScanType, eAspectRatio AspectRatio);
   void SetFileName(const char *FileName);
   int Errors(void) const { return errors; } // returns -1 if undefined
-  void SetErrors(int Errors);
+  int TmpErrors(void) const { return tmpErrors; } // returns -1 if undefined
+  void SetErrors(int Errors, int TmpErrors = 0);
   bool Write(FILE *f, const char *Prefix = "") const;
   bool Read(bool Force = false);
   bool Write(void) const;
@@ -150,7 +152,8 @@ public:
   int Priority(void) const { return info->Priority(); }
   int Lifetime(void) const { return info->Lifetime(); }
   time_t Deleted(void) const { return deleted; }
-  void SetDeleted(void) { deleted = time(NULL); }
+  bool RetentionExpired(void) const;
+  void SetDeleted(void);
   virtual int Compare(const cListObject &ListObject) const override;
   bool IsInPath(const char *Path) const;
        ///< Returns true if this recording is stored anywhere under the given Path.
@@ -502,7 +505,10 @@ private:
   void ConvertToPes(tIndexTs *IndexTs, int Count);
   bool CatchUp(int Index = -1);
 public:
-  cIndexFile(const char *FileName, bool Record, bool IsPesRecording = false, bool PauseLive = false, bool Update = false);
+  cIndexFile(const char *FileName, bool Record, bool IsPesRecording = false, bool PauseLive = false);
+  [[deprecated("use cIndexFile(::cIndexFile(const char *, bool, bool, bool) instead")]]
+  cIndexFile(const char *FileName, bool Record, bool IsPesRecording, bool PauseLive, bool Update)
+  :cIndexFile(FileName, Record, IsPesRecording, PauseLive) {}
   ~cIndexFile();
   bool Ok(void) { return index != NULL; }
   bool Write(bool Independent, uint16_t FileNumber, off_t FileOffset, bool Errors = false, bool Missing = false);
@@ -541,6 +547,7 @@ public:
   ~cFileName();
   const char *Name(void) { return fileName; }
   uint16_t Number(void) { return fileNumber; }
+  [[deprecated("will be removed in a future version, if your plugin uses this function, contact vdr@tvdr.de")]]
   bool GetLastPatPmtVersions(int &PatVersion, int &PmtVersion);
   cUnbufferedFile *Open(void);
   void Close(void);
@@ -577,11 +584,11 @@ char *ExchangeChars(char *s, bool ToFileSystem);
       // be modified and may be reallocated if more space is needed. The return
       // value points to the resulting string, which may be different from s.
 
-bool GenerateIndex(const char *FileName, bool Update = false);
+bool GenerateIndex(const char *FileName);
        ///< Generates the index of the existing recording with the given FileName.
-       ///< If Update is true, an existing index file will be checked whether it is
-       ///< complete, and will be updated if it isn't. Otherwise an existing index
-       ///< file will be removed before a new one is generated.
+       ///< An existing index file will be removed before a new one is generated.
+[[deprecated("use GenerateIndex(const char *FileName) instead")]]
+inline bool GenerateIndex(const char *FileName, bool Update) { return GenerateIndex(FileName); }
 
 enum eRecordingsSortDir { rsdAscending, rsdDescending };
 enum eRecordingsSortMode { rsmName, rsmTime };
