@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: sections.h 1.5 2005/08/13 11:23:55 kls Exp $
+ * $Id: sections.h 5.2 2025/03/02 11:03:35 kls Exp $
  */
 
 #ifndef __SECTIONS_H
@@ -27,15 +27,17 @@ private:
   cDevice *device;
   int statusCount;
   bool on, waitForLock;
-  time_t lastIncompleteSection;
+  bool flush;
+  bool startFilters;
+  cTimeMs flushTimer;
   cList<cFilter> filters;
   cList<cFilterHandle> filterHandles;
   void Add(const cFilterData *FilterData);
   void Del(const cFilterData *FilterData);
-  virtual void Action(void);
+  virtual void Action(void) override;
 public:
   cSectionHandler(cDevice *Device);
-  virtual ~cSectionHandler();
+  virtual ~cSectionHandler() override;
   int Source(void);
   int Transponder(void);
   const cChannel *Channel(void);
