@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skins.c 1.14 2008/02/17 11:31:09 kls Exp $
+ * $Id: skins.c 5.6 2025/03/02 11:03:35 kls Exp $
  */
 
 #include "skins.h"
@@ -27,7 +27,7 @@ private:
   cCondVar condVar;
 public:
   cSkinQueuedMessage(eMessageType Type, const char *s, int Seconds, int Timeout);
-  virtual ~cSkinQueuedMessage();
+  virtual ~cSkinQueuedMessage() override;
   };
 
 cSkinQueuedMessage::cSkinQueuedMessage(eMessageType Type, const char *s, int Seconds, int Timeout)
@@ -63,13 +63,35 @@ cSkinDisplay::~cSkinDisplay()
   current = NULL;
 }
 
+// --- cSkinDisplayChannel ---------------------------------------------------
+
+cSkinDisplayChannel::cSkinDisplayChannel(void)
+{
+  positioner = NULL;
+}
+
+void cSkinDisplayChannel::SetPositioner(const cPositioner *Positioner)
+{
+  if (positioner && Positioner != positioner)
+     SetMessage(mtInfo, NULL);
+  positioner = Positioner;
+  if (positioner)
+     SetMessage(mtInfo, cString::sprintf(tr("Moving dish to %.1f..."), double(positioner->TargetLongitude()) / 10));
+}
+
 // --- cSkinDisplayMenu ------------------------------------------------------
 
 cSkinDisplayMenu::cSkinDisplayMenu(void)
 {
+  menuCategory = mcUndefined;
   SetTabs(0);
 }
 
+void cSkinDisplayMenu::SetMenuCategory(eMenuCategory MenuCategory)
+{
+  menuCategory = MenuCategory;
+}
+
 void cSkinDisplayMenu::SetTabs(int Tab1, int Tab2, int Tab3, int Tab4, int Tab5)
 {
   tabs[0] = 0;
@@ -79,7 +101,7 @@ void cSkinDisplayMenu::SetTabs(int Tab1, int Tab2, int Tab3, int Tab4, int Tab5)
   tabs[4] = Tab4 ? tabs[3] + Tab4 : 0;
   tabs[5] = Tab5 ? tabs[4] + Tab5 : 0;
   for (int i = 1; i < MaxTabs; i++)
-      tabs[i] *= 12;//XXX average character width of font used for items - see also skincurses.c!!!
+      tabs[i] *= AvgCharWidth();
 }
 
 void cSkinDisplayMenu::Scroll(bool Up, bool Page)
@@ -117,15 +139,20 @@ int cSkinDisplayMenu::GetTextAreaWidth(void) const
   return 0;
 }
 
-const cFont *cSkinDisplayMenu::GetTextAreaFont(bool) const
+const cFont *cSkinDisplayMenu::GetTextAreaFont(bool  FixedFont) const
 {
-  return NULL;
+  return cFont::GetFont(FixedFont ? fontFix : fontOsd);
 }
 
 // --- cSkinDisplayReplay::cProgressBar --------------------------------------
 
 cSkinDisplayReplay::cProgressBar::cProgressBar(int Width, int Height, int Current, int Total, const cMarks *Marks, tColor ColorSeen, tColor ColorRest, tColor ColorSelected, tColor ColorMark, tColor ColorCurrent)
-:cBitmap(Width, Height, 2)
+:cSkinDisplayReplay::cProgressBar::cProgressBar(Width, Height, Current, Total, Marks, NULL, ColorSeen, ColorRest, ColorSelected, ColorMark, ColorCurrent, clrBlack)
+{
+}
+
+cSkinDisplayReplay::cProgressBar::cProgressBar(int Width, int Height, int Current, int Total, const cMarks *Marks, const cErrors *Errors, tColor ColorSeen, tColor ColorRest, tColor ColorSelected, tColor ColorMark, tColor ColorCurrent, tColor ColorError)
+:cBitmap(Width, Height, 4)
 {
   total = Total;
   if (total > 0) {
@@ -135,17 +162,27 @@ cSkinDisplayReplay::cProgressBar::cProgressBar(int Width, int Height, int Curren
      if (Marks) {
         bool Start = true;
         for (const cMark *m = Marks->First(); m; m = Marks->Next(m)) {
-            int p1 = Pos(m->position);
+            int p1 = Pos(m->Position());
             if (Start) {
                const cMark *m2 = Marks->Next(m);
-               int p2 = Pos(m2 ? m2->position : total);
+               int p2 = Pos(m2 ? m2->Position() : total);
                int h = Height / 3;
-               DrawRectangle(p1, h, p2, Height - h, ColorSelected);
+               DrawRectangle(p1, h, p2, Height - h - 1, ColorSelected);
                }
-            Mark(p1, Start, m->position == Current, ColorMark, ColorCurrent);
+            Mark(p1, Start, m->Position() == Current, ColorMark, ColorCurrent);
             Start = !Start;
             }
         }
+     if (Errors) {
+        int LastPos = -1;
+        for (int i = 0; i < Errors->Size(); i++) {
+            int p1 = Errors->At(i);
+            if (p1 != LastPos) {
+               Error(Pos(Errors->At(i)), ColorError);
+               LastPos = p1;
+               }
+            }
+        }
      }
 }
 
@@ -159,11 +196,30 @@ void cSkinDisplayReplay::cProgressBar::Mark(int x, bool Start, bool Current, tCo
       }
 }
 
+void cSkinDisplayReplay::cProgressBar::Error(int x, tColor ColorError)
+{
+  const int d = (Height() / 9) & ~0x01; // must be even
+  const int h = Height() / 2;
+  const int e = Height() / 4;
+  DrawRectangle(x, e, x, Height() -e - 1, ColorError);
+  DrawRectangle(x - d, h, x + d, h, ColorError);
+  for (int i = 1; i <= d; i++) {
+      DrawRectangle(x - d + i, h - i, x + d - i, h - i, ColorError);
+      DrawRectangle(x - d + i, h + i, x + d - i, h + i, ColorError);
+      }
+}
+
 // --- cSkinDisplayReplay ----------------------------------------------------
 
 cSkinDisplayReplay::cSkinDisplayReplay(void)
 {
   marks = NULL;
+  errors = NULL;
+}
+
+void cSkinDisplayReplay::SetRecording(const cRecording *Recording)
+{
+  SetTitle(Recording->Title());
 }
 
 void cSkinDisplayReplay::SetMarks(const cMarks *Marks)
@@ -171,6 +227,11 @@ void cSkinDisplayReplay::SetMarks(const cMarks *Marks)
   marks = Marks;
 }
 
+void cSkinDisplayReplay::SetErrors(const cErrors *Errors)
+{
+  errors = Errors;
+}
+
 // --- cSkin -----------------------------------------------------------------
 
 cSkin::cSkin(const char *Name, cTheme *Theme)
@@ -222,6 +283,13 @@ bool cSkins::SetCurrent(const char *Name)
 
 eKeys cSkins::Message(eMessageType Type, const char *s, int Seconds)
 {
+  if (!cThread::IsMainThread()) {
+     if (Type != mtStatus)
+        QueueMessage(Type, s, Seconds);
+     else
+        dsyslog("cSkins::Message(%d, \"%s\", %d) called from background thread - ignored! (Use cSkins::QueueMessage() instead)", Type, s, Seconds);
+     return kNone;
+     }
   switch (Type) {
     case mtInfo:    isyslog("info: %s", s); break;
     case mtWarning: isyslog("warning: %s", s); break;
@@ -237,7 +305,7 @@ eKeys cSkins::Message(eMessageType Type, const char *s, int Seconds)
      }
   cSkinDisplay::Current()->SetMessage(Type, s);
   cSkinDisplay::Current()->Flush();
-  cStatus::MsgOsdStatusMessage(s);
+  cStatus::MsgOsdStatusMessage(Type, s);
   eKeys k = kNone;
   if (Type != mtStatus) {
      k = Interface->Wait(Seconds);
@@ -248,7 +316,7 @@ eKeys cSkins::Message(eMessageType Type, const char *s, int Seconds)
         }
      else {
         cSkinDisplay::Current()->SetMessage(Type, NULL);
-        cStatus::MsgOsdStatusMessage(NULL);
+        cStatus::MsgOsdStatusMessage(Type, NULL);
         }
      }
   else if (!s && displayMessage) {
@@ -321,6 +389,14 @@ void cSkins::ProcessQueuedMessages(void)
      dsyslog("cSkins::ProcessQueuedMessages() called from background thread - ignored!");
      return;
      }
+  // Check whether there is a cSkinDisplay object (if any) that implements SetMessage():
+  if (cSkinDisplay *sd = cSkinDisplay::Current()) {
+     if (!(dynamic_cast<cSkinDisplayChannel *>(sd) ||
+           dynamic_cast<cSkinDisplayMenu *>(sd) ||
+           dynamic_cast<cSkinDisplayReplay *>(sd) ||
+           dynamic_cast<cSkinDisplayMessage *>(sd)))
+        return;
+     }
   cSkinQueuedMessage *msg = NULL;
   // Get the first waiting message:
   queueMessageMutex.Lock();
