@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 5.13 2025/07/21 08:39:20 kls Exp $
+ * $Id: svdrp.c 5.15 2026/01/14 10:19:11 kls Exp $
  */
 
 #include "svdrp.h"
@@ -948,7 +948,9 @@ const char *HelpPages[] = {
   "    by the LSTC command.",
   "NEWT <settings>\n"
   "    Create a new timer. Settings must be in the same format as returned\n"
-  "    by the LSTT command.",
+  "    by the LSTT command. If a timer with the same channel, day, start\n"
+  "    and stop time already exists, the data of the existing timer is returned\n"
+  "    with code 550.",
   "NEXT [ abs | rel ]\n"
   "    Show the next timer event. If no option is given, the output will be\n"
   "    in human readable form. With option 'abs' the absolute time of the next\n"
@@ -963,7 +965,7 @@ const char *HelpPages[] = {
   "    Used by peer-to-peer connections between VDRs to keep the connection\n"
   "    from timing out. May be used at any time and simply returns a line of\n"
   "    the form '<hostname> is alive'.",
-  "PLAY <id> [ begin | <position> ]\n"
+  "PLAY [ <id> [ begin | <position> ] ]\n"
   "    Play the recording with the given id. Before a recording can be\n"
   "    played, an LSTR command should have been executed in order to retrieve\n"
   "    the recording ids.\n"
@@ -972,7 +974,9 @@ const char *HelpPages[] = {
   "    position. If neither 'begin' nor a <position> are given, replay is resumed\n"
   "    at the position where any previous replay was stopped, or from the beginning\n"
   "    by default. To control or stop the replay session, use the usual remote\n"
-  "    control keypresses via the HITK command.",
+  "    control keypresses via the HITK command.\n"
+  "    Without any parameters PLAY returns the id and title of the recording that\n"
+  "    is currently being played (if any).",
   "PLUG <name> [ help | main ] [ <command> [ <options> ]]\n"
   "    Send a command to a plugin.\n"
   "    The PLUG command without any parameters lists all plugins.\n"
@@ -2273,10 +2277,19 @@ void cSVDRPServer::CmdNEWT(const char *Option)
      cTimer *Timer = new cTimer;
      if (Timer->Parse(Option)) {
         LOCK_TIMERS_WRITE;
-        Timer->ClrFlags(tfRecording);
-        Timers->Add(Timer);
-        isyslog("SVDRP %s < %s added timer %s", Setup.SVDRPHostName, *clientName, *Timer->ToDescr());
-        Reply(250, "%d %s", Timer->Id(), *Timer->ToText(true));
+        const cTimer *t = Timers->GetTimer(Timer);
+        if (!t) {
+           Timer->ClrFlags(tfRecording);
+           Timers->Add(Timer);
+           isyslog("SVDRP %s < %s added timer %s", Setup.SVDRPHostName, *clientName, *Timer->ToDescr());
+           Reply(250, "%d %s", Timer->Id(), *Timer->ToText(true));
+           }
+        else {
+           isyslog("SVDRP %s < %s attempted to add timer %s", Setup.SVDRPHostName, *clientName, *Timer->ToDescr());
+           isyslog("SVDRP %s < %s timer already exists as %s", Setup.SVDRPHostName, *clientName, *t->ToDescr());
+           delete Timer;
+           Reply(550, "%d %s", t->Id(), *t->ToText(true));
+           }
         return;
         }
      else
@@ -2359,8 +2372,15 @@ void cSVDRPServer::CmdPLAY(const char *Option)
         Reply(501, "Error in recording id \"%s\"", num);
      free(opt);
      }
+  else if (const char *FileName = cReplayControl::NowReplaying()) {
+     LOCK_RECORDINGS_READ;
+     if (const cRecording *Recording = Recordings->GetByName(FileName))
+        Reply(250, "%d %s", Recording->Id(), Recording->Title(' ', true));
+     else
+        Reply(550, "Recording \"%s\" not found", FileName);
+     }
   else
-     Reply(501, "Missing recording id");
+     Reply(550, "Not playing");
 }
 
 void cSVDRPServer::CmdPLUG(const char *Option)
