@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 5.18 2025/06/23 09:23:05 kls Exp $
+ * $Id: tools.c 5.20 2025/12/31 12:47:03 kls Exp $
  */
 
 #include "tools.h"
@@ -407,6 +407,11 @@ bool StrInArray(const char *a[], const char *s)
   return false;
 }
 
+cString Indent(int n, const char *s)
+{
+  return cString::sprintf("%*s%s", n, "", s);
+}
+
 cString AddDirectory(const char *DirName, const char *FileName)
 {
   if (*FileName == '/')
@@ -756,10 +761,9 @@ off_t FileSize(const char *FileName)
 
 cTimeMs::cTimeMs(int Ms)
 {
-  if (Ms >= 0)
-     Set(Ms);
-  else
-     begin = 0;
+  begin = 0;
+  end = 0;
+  Set(Ms);
 }
 
 uint64_t cTimeMs::Now(void)
@@ -807,12 +811,15 @@ uint64_t cTimeMs::Now(void)
 
 void cTimeMs::Set(int Ms)
 {
-  begin = Now() + Ms;
+  if (Ms >= 0) {
+     begin = Now();
+     end = begin + Ms;
+     }
 }
 
 bool cTimeMs::TimedOut(void) const
 {
-  return Now() >= begin;
+  return Now() >= end;
 }
 
 uint64_t cTimeMs::Elapsed(void) const
@@ -820,6 +827,16 @@ uint64_t cTimeMs::Elapsed(void) const
   return Now() - begin;
 }
 
+uint64_t cTimeMs::Remaining(void) const
+{
+  return end - Now();
+}
+
+void cTimeMs::Reset(void)
+{
+  Set(end - begin);
+}
+
 // --- UTF-8 support ---------------------------------------------------------
 
 static uint SystemToUtf8[128] = { 0 };
